from django.contrib import admin

# Register your models here.
from .models import SiteConfigModel

class SiteConfigModelAdmin(admin.ModelAdmin):
    list_display = ["key", "site"]
    search_fields = ["key", "site__name"]


admin.site.register(SiteConfigModel, SiteConfigModelAdmin)



from django.test import Client, TestCase
from django.urls import reverse
from core.config import ExampleClass
from .models import SiteConfigModel

class ExampleTests(TestCase):
    
    def setUp(self):
        self.client = Client()
    
    def test_default_value(self):
        default = ExampleClass().get_key_value()
        self.assertEquals(default, {"example": "Default Value"})

    def test_create_config(self):
        self.assertEqual(SiteConfigModel.objects.count(), 0)
        uri = reverse("core-example")
        response = self.client.post(uri, {"example": "Testing"})
        self.assertEqual(SiteConfigModel.objects.count(), 1)
        config = SiteConfigModel.objects.last()
        self.assertEqual(config.key, "core.config.ExampleClass")
        self.assertEqual(config.value, {"example": "Testing"})
        self.assertEqual(config.value, ExampleClass().get_key_value())
        self.assertEqual(config.value["example"], ExampleClass().get_key_value("example"))

    def test_save(self):
        example = ExampleClass()
        self.assertEqual(example.instance, None)
        example.save("new value", "test key")
        setting = SiteConfigModel.objects.last()
        self.assertEqual(example.instance, setting)
        self.assertEqual(example.value, setting.value)
        self.assertEqual(example.value, {"test key": "new value"})

    def test_save_via_dict(self):
        example = ExampleClass()
        test_data = {"hello": "world", "test": "value"}
        example.save(test_data)
        self.assertEqual(example.value, test_data)
        self.assertEqual(example.get_key_value("hello"), "world")
        self.assertEqual(example.get_key_value("test"), "value")

    def test_delete(self):
        ExampleClass().save("Testing", "example")
        self.assertEqual(SiteConfigModel.objects.count(), 1)
        ExampleClass().delete()
        self.assertEqual(ExampleClass().instance, None)
        self.assertEqual(ExampleClass().value, dict())
        self.assertEqual(SiteConfigModel.objects.count(), 0)



# Utility to make sure to get the correct site object to use
from django.contrib.sites.models import Site


def get_current_site(request):
    return getattr(request, "site", Site.objects.get_current())



# Generated by Django 3.1.12 on 2021-06-25 23:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteConfigKeyModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=120, unique=True, verbose_name='Key Label')),
                ('key', models.SlugField(blank=True, editable=False, unique=True, verbose_name='Key')),
                ('schema', models.JSONField(blank=True, default=dict, null=True, verbose_name='Schema')),
                ('deleted', models.BooleanField(default=False, help_text='Soft Deleted?', verbose_name='Deleted')),
                ('multiples', models.BooleanField(help_text='Are multiple setting values allowed?', verbose_name='Multiples')),
                ('admin_only', models.BooleanField(help_text='Are these only allowed for people with Django Admin privilages?', verbose_name='Django Admin Only')),
                ('setting_type', models.IntegerField(choices=[(1, 'Common'), (2, 'Response Meta Tag')], verbose_name='Setting Type')),
            ],
            options={
                'verbose_name': 'Site Setting Key',
                'verbose_name_plural': 'Site Setting Keys',
            },
        ),
        migrations.CreateModel(
            name='SiteConfigModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('deleted', models.BooleanField(default=False, help_text='Soft Delete the Setting value', verbose_name='Deleted')),
                ('value', models.JSONField(default=dict, verbose_name='Value')),
                ('key', models.ForeignKey(help_text='What setting is this value for?', on_delete=django.db.models.deletion.CASCADE, to='siteconfigs.siteconfigkeymodel', verbose_name='Site Setting Key Name')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='settings', to='sites.site')),
            ],
            options={
                'verbose_name': 'Site Setting',
                'verbose_name_plural': 'Site Settings',
            },
        ),
    ]



# Generated by Django 3.1.13 on 2021-08-18 23:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('siteconfigs', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='siteconfigmodel',
            name='key',
            field=models.CharField(max_length=120, verbose_name='Key'),
        ),
        migrations.AddConstraint(
            model_name='siteconfigmodel',
            constraint=models.UniqueConstraint(fields=('key', 'site'), name='unique key per site'),
        ),
        migrations.DeleteModel(
            name='SiteConfigKeyModel',
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from .models import SiteConfigModel

class SiteConfigBaseClass():
    key = None
    label = None
    site = None
    instance = None
    value = dict()
    default = dict()
    form_class = None
    is_superuser_config = False

    def __init__(self, site, key):
        self.site = site
        try:
            site_conf = SiteConfigModel.objects.get(site=site, key=key)
            self.instance = site_conf
            self.value = site_conf.value
        except ObjectDoesNotExist:
            pass

    def get_key_value(self, key=None):
        if key:
            return self.value.get(key, self.default.get(key))
        elif self.value:
            return self.value
        return self.default

    # can pass in a dictionary or a single key and value
    # easier to accomodate settings with multiple values
    def save(self, data, value_key = None):
        config, created = SiteConfigModel.objects.get_or_create(site=self.site, key=self.key)
        if value_key:
            config.value.update({
                value_key: data
            })
        elif isinstance(data, dict):
            config.value.update(data)
        config.save()
        self.instance = config
        self.value = config.value

    def delete(self):
        if self.instance:
            self.instance.delete()
            self.instance = None
            self.value = dict()

    



# import uuid

# from django.conf import settings
from django.db import models
from django.utils.translation import gettext as _
from django.utils.text import slugify

# from django.urls import reverse
from django.contrib.sites.models import Site


class SiteConfigModel(models.Model):
    key = models.CharField(_("Key"), max_length=120, blank=False) # Path to module
    site = models.ForeignKey(
        Site,
        on_delete=models.CASCADE,
        related_name="settings",
    )  # This uses a callable so it will not trigger a migration with the projects it's included in
    deleted = models.BooleanField(
        _("Deleted"), default=False, help_text=_("Soft Delete the Setting value")
    )
    value = models.JSONField(_("Value"), default=dict)

    # TODO: Create manager to get record by key slug
    #       ideally it would look something like Site.config.get("some-key")
    #       it should return a value or None, no matter wether or not the key is set for that site

    class Meta:
        verbose_name = _("Site Setting")
        verbose_name_plural = _("Site Settings")
        constraints = [
            models.UniqueConstraint(fields=["key", "site"], name="unique key per site")
        ]

    def __str__(self):
        return "{key} ({site}): {value}".format(
            key=self.key, site=self.site.name, value=str(self.value)
        )



# --------------------------------------------
# Copyright 2021, Grant Viklund
# @Author: Grant Viklund
# @Date:   2021-06-25 14:51:10
# --------------------------------------------

from os import path
from setuptools import setup, find_packages

from siteconfigs.__version__ import VERSION

readme_file = path.join(path.dirname(path.abspath(__file__)), "README.md")

try:
    from m2r import parse_from_file

    long_description = parse_from_file(readme_file)  # Convert the file to RST for PyPI
except ImportError:
    # m2r may not be installed in user environment
    with open(readme_file) as f:
        long_description = f.read()

package_metadata = {
    "name": "django-site-configs",
    "version": VERSION,
    "description": "A Django app that manages configurations per-site on a multi-site setup.  These configs can be made avaialble to users of different sites or administerd via the Django admin.",
    "long_description": long_description,
    "url": "https://github.com/renderbox/django-site-configs/",
    "author": "Grant Viklund",
    "author_email": "renderbox@gmail.com",
    "license": "MIT license",
    "classifiers": [
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    "keywords": ["django", "app"],
}

setup(
    **package_metadata,
    packages=find_packages(),
    package_data={"siteconfigs": ["templates/siteconfigs/*.html"]},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        "Django>=3.2,<4.1",
        "djangorestframework",
    ],
    extras_require={
        "dev": [  # Packages needed by developers
            "django-crispy-forms",
            "django-allauth",
            "django-extensions",
            "ipython",
            "black",
        ],
        "test": [],  # Packages needed to run tests
        "prod": [],  # Packages needed to run in the deployment
        "build": [  # Packages needed to build the package
            "setuptools",
            "wheel",
            "twine",
            "m2r",
        ],
        "docs": [  # Packages needed to generate docs
            "recommonmark",
            "m2r",
            "django_extensions",
            "coverage",
            "Sphinx",
            "rstcheck",
            "sphinx-rtd-theme",  # Assumes a Read The Docs theme for opensource projects
        ],
    }
)
